<?php
/*
 * Copyright 2024 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * GENERATED CODE WARNING
 * Generated by gapic-generator-php from the file
 * https://github.com/googleapis/googleapis/blob/master/google/cloud/binaryauthorization/v1/service.proto
 * Updates to the above are reflected here through a refresh process.
 */

namespace Google\Cloud\BinaryAuthorization\V1\Client;

use Google\ApiCore\ApiException;
use Google\ApiCore\CredentialsWrapper;
use Google\ApiCore\GapicClientTrait;
use Google\ApiCore\PagedListResponse;
use Google\ApiCore\ResourceHelperTrait;
use Google\ApiCore\RetrySettings;
use Google\ApiCore\Transport\TransportInterface;
use Google\ApiCore\ValidationException;
use Google\Auth\FetchAuthTokenInterface;
use Google\Cloud\BinaryAuthorization\V1\Attestor;
use Google\Cloud\BinaryAuthorization\V1\CreateAttestorRequest;
use Google\Cloud\BinaryAuthorization\V1\DeleteAttestorRequest;
use Google\Cloud\BinaryAuthorization\V1\GetAttestorRequest;
use Google\Cloud\BinaryAuthorization\V1\GetPolicyRequest;
use Google\Cloud\BinaryAuthorization\V1\ListAttestorsRequest;
use Google\Cloud\BinaryAuthorization\V1\Policy;
use Google\Cloud\BinaryAuthorization\V1\UpdateAttestorRequest;
use Google\Cloud\BinaryAuthorization\V1\UpdatePolicyRequest;
use GuzzleHttp\Promise\PromiseInterface;

/**
 * Service Description: Google Cloud Management Service for Binary Authorization admission policies
 * and attestation authorities.
 *
 * This API implements a REST model with the following objects:
 *
 * * [Policy][google.cloud.binaryauthorization.v1.Policy]
 * * [Attestor][google.cloud.binaryauthorization.v1.Attestor]
 *
 * This class provides the ability to make remote calls to the backing service through method
 * calls that map to API methods.
 *
 * Many parameters require resource names to be formatted in a particular way. To
 * assist with these names, this class includes a format method for each type of
 * name, and additionally a parseName method to extract the individual identifiers
 * contained within formatted names that are returned by the API.
 *
 * @method PromiseInterface createAttestorAsync(CreateAttestorRequest $request, array $optionalArgs = [])
 * @method PromiseInterface deleteAttestorAsync(DeleteAttestorRequest $request, array $optionalArgs = [])
 * @method PromiseInterface getAttestorAsync(GetAttestorRequest $request, array $optionalArgs = [])
 * @method PromiseInterface getPolicyAsync(GetPolicyRequest $request, array $optionalArgs = [])
 * @method PromiseInterface listAttestorsAsync(ListAttestorsRequest $request, array $optionalArgs = [])
 * @method PromiseInterface updateAttestorAsync(UpdateAttestorRequest $request, array $optionalArgs = [])
 * @method PromiseInterface updatePolicyAsync(UpdatePolicyRequest $request, array $optionalArgs = [])
 */
final class BinauthzManagementServiceV1Client
{
    use GapicClientTrait;
    use ResourceHelperTrait;

    /** The name of the service. */
    private const SERVICE_NAME = 'google.cloud.binaryauthorization.v1.BinauthzManagementServiceV1';

    /**
     * The default address of the service.
     *
     * @deprecated SERVICE_ADDRESS_TEMPLATE should be used instead.
     */
    private const SERVICE_ADDRESS = 'binaryauthorization.googleapis.com';

    /** The address template of the service. */
    private const SERVICE_ADDRESS_TEMPLATE = 'binaryauthorization.UNIVERSE_DOMAIN';

    /** The default port of the service. */
    private const DEFAULT_SERVICE_PORT = 443;

    /** The name of the code generator, to be included in the agent header. */
    private const CODEGEN_NAME = 'gapic';

    /** The default scopes required by the service. */
    public static $serviceScopes = ['https://www.googleapis.com/auth/cloud-platform'];

    private static function getClientDefaults()
    {
        return [
            'serviceName' => self::SERVICE_NAME,
            'apiEndpoint' => self::SERVICE_ADDRESS . ':' . self::DEFAULT_SERVICE_PORT,
            'clientConfig' => __DIR__ . '/../resources/binauthz_management_service_v1_client_config.json',
            'descriptorsConfigPath' => __DIR__ . '/../resources/binauthz_management_service_v1_descriptor_config.php',
            'gcpApiConfigPath' => __DIR__ . '/../resources/binauthz_management_service_v1_grpc_config.json',
            'credentialsConfig' => [
                'defaultScopes' => self::$serviceScopes,
            ],
            'transportConfig' => [
                'rest' => [
                    'restClientConfigPath' =>
                        __DIR__ . '/../resources/binauthz_management_service_v1_rest_client_config.php',
                ],
            ],
        ];
    }

    /**
     * Formats a string containing the fully-qualified path to represent a attestor
     * resource.
     *
     * @param string $project
     * @param string $attestor
     *
     * @return string The formatted attestor resource.
     */
    public static function attestorName(string $project, string $attestor): string
    {
        return self::getPathTemplate('attestor')->render([
            'project' => $project,
            'attestor' => $attestor,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * location_policy resource.
     *
     * @param string $location
     *
     * @return string The formatted location_policy resource.
     */
    public static function locationPolicyName(string $location): string
    {
        return self::getPathTemplate('locationPolicy')->render([
            'location' => $location,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a policy
     * resource.
     *
     * @param string $project
     *
     * @return string The formatted policy resource.
     */
    public static function policyName(string $project): string
    {
        return self::getPathTemplate('policy')->render([
            'project' => $project,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a project
     * resource.
     *
     * @param string $project
     *
     * @return string The formatted project resource.
     */
    public static function projectName(string $project): string
    {
        return self::getPathTemplate('project')->render([
            'project' => $project,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * project_policy resource.
     *
     * @param string $project
     *
     * @return string The formatted project_policy resource.
     */
    public static function projectPolicyName(string $project): string
    {
        return self::getPathTemplate('projectPolicy')->render([
            'project' => $project,
        ]);
    }

    /**
     * Parses a formatted name string and returns an associative array of the components in the name.
     * The following name formats are supported:
     * Template: Pattern
     * - attestor: projects/{project}/attestors/{attestor}
     * - locationPolicy: locations/{location}/policy
     * - policy: projects/{project}/policy
     * - project: projects/{project}
     * - projectPolicy: projects/{project}/policy
     *
     * The optional $template argument can be supplied to specify a particular pattern,
     * and must match one of the templates listed above. If no $template argument is
     * provided, or if the $template argument does not match one of the templates
     * listed, then parseName will check each of the supported templates, and return
     * the first match.
     *
     * @param string $formattedName The formatted name string
     * @param string $template      Optional name of template to match
     *
     * @return array An associative array from name component IDs to component values.
     *
     * @throws ValidationException If $formattedName could not be matched.
     */
    public static function parseName(string $formattedName, string $template = null): array
    {
        return self::parseFormattedName($formattedName, $template);
    }

    /**
     * Constructor.
     *
     * @param array $options {
     *     Optional. Options for configuring the service API wrapper.
     *
     *     @type string $apiEndpoint
     *           The address of the API remote host. May optionally include the port, formatted
     *           as "<uri>:<port>". Default 'binaryauthorization.googleapis.com:443'.
     *     @type string|array|FetchAuthTokenInterface|CredentialsWrapper $credentials
     *           The credentials to be used by the client to authorize API calls. This option
     *           accepts either a path to a credentials file, or a decoded credentials file as a
     *           PHP array.
     *           *Advanced usage*: In addition, this option can also accept a pre-constructed
     *           {@see \Google\Auth\FetchAuthTokenInterface} object or
     *           {@see \Google\ApiCore\CredentialsWrapper} object. Note that when one of these
     *           objects are provided, any settings in $credentialsConfig will be ignored.
     *     @type array $credentialsConfig
     *           Options used to configure credentials, including auth token caching, for the
     *           client. For a full list of supporting configuration options, see
     *           {@see \Google\ApiCore\CredentialsWrapper::build()} .
     *     @type bool $disableRetries
     *           Determines whether or not retries defined by the client configuration should be
     *           disabled. Defaults to `false`.
     *     @type string|array $clientConfig
     *           Client method configuration, including retry settings. This option can be either
     *           a path to a JSON file, or a PHP array containing the decoded JSON data. By
     *           default this settings points to the default client config file, which is
     *           provided in the resources folder.
     *     @type string|TransportInterface $transport
     *           The transport used for executing network requests. May be either the string
     *           `rest` or `grpc`. Defaults to `grpc` if gRPC support is detected on the system.
     *           *Advanced usage*: Additionally, it is possible to pass in an already
     *           instantiated {@see \Google\ApiCore\Transport\TransportInterface} object. Note
     *           that when this object is provided, any settings in $transportConfig, and any
     *           $apiEndpoint setting, will be ignored.
     *     @type array $transportConfig
     *           Configuration options that will be used to construct the transport. Options for
     *           each supported transport type should be passed in a key for that transport. For
     *           example:
     *           $transportConfig = [
     *               'grpc' => [...],
     *               'rest' => [...],
     *           ];
     *           See the {@see \Google\ApiCore\Transport\GrpcTransport::build()} and
     *           {@see \Google\ApiCore\Transport\RestTransport::build()} methods for the
     *           supported options.
     *     @type callable $clientCertSource
     *           A callable which returns the client cert as a string. This can be used to
     *           provide a certificate and private key to the transport layer for mTLS.
     * }
     *
     * @throws ValidationException
     */
    public function __construct(array $options = [])
    {
        $clientOptions = $this->buildClientOptions($options);
        $this->setClientOptions($clientOptions);
    }

    /** Handles execution of the async variants for each documented method. */
    public function __call($method, $args)
    {
        if (substr($method, -5) !== 'Async') {
            trigger_error('Call to undefined method ' . __CLASS__ . "::$method()", E_USER_ERROR);
        }

        array_unshift($args, substr($method, 0, -5));
        return call_user_func_array([$this, 'startAsyncCall'], $args);
    }

    /**
     * Creates an [attestor][google.cloud.binaryauthorization.v1.Attestor], and returns a copy of the new
     * [attestor][google.cloud.binaryauthorization.v1.Attestor]. Returns NOT_FOUND if the project does not exist,
     * INVALID_ARGUMENT if the request is malformed, ALREADY_EXISTS if the
     * [attestor][google.cloud.binaryauthorization.v1.Attestor] already exists.
     *
     * The async variant is
     * {@see BinauthzManagementServiceV1Client::createAttestorAsync()} .
     *
     * @example samples/V1/BinauthzManagementServiceV1Client/create_attestor.php
     *
     * @param CreateAttestorRequest $request     A request to house fields associated with the call.
     * @param array                 $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return Attestor
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function createAttestor(CreateAttestorRequest $request, array $callOptions = []): Attestor
    {
        return $this->startApiCall('CreateAttestor', $request, $callOptions)->wait();
    }

    /**
     * Deletes an [attestor][google.cloud.binaryauthorization.v1.Attestor]. Returns NOT_FOUND if the
     * [attestor][google.cloud.binaryauthorization.v1.Attestor] does not exist.
     *
     * The async variant is
     * {@see BinauthzManagementServiceV1Client::deleteAttestorAsync()} .
     *
     * @example samples/V1/BinauthzManagementServiceV1Client/delete_attestor.php
     *
     * @param DeleteAttestorRequest $request     A request to house fields associated with the call.
     * @param array                 $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function deleteAttestor(DeleteAttestorRequest $request, array $callOptions = []): void
    {
        $this->startApiCall('DeleteAttestor', $request, $callOptions)->wait();
    }

    /**
     * Gets an [attestor][google.cloud.binaryauthorization.v1.Attestor].
     * Returns NOT_FOUND if the [attestor][google.cloud.binaryauthorization.v1.Attestor] does not exist.
     *
     * The async variant is
     * {@see BinauthzManagementServiceV1Client::getAttestorAsync()} .
     *
     * @example samples/V1/BinauthzManagementServiceV1Client/get_attestor.php
     *
     * @param GetAttestorRequest $request     A request to house fields associated with the call.
     * @param array              $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return Attestor
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function getAttestor(GetAttestorRequest $request, array $callOptions = []): Attestor
    {
        return $this->startApiCall('GetAttestor', $request, $callOptions)->wait();
    }

    /**
     * A [policy][google.cloud.binaryauthorization.v1.Policy] specifies the [attestors][google.cloud.binaryauthorization.v1.Attestor] that must attest to
     * a container image, before the project is allowed to deploy that
     * image. There is at most one policy per project. All image admission
     * requests are permitted if a project has no policy.
     *
     * Gets the [policy][google.cloud.binaryauthorization.v1.Policy] for this project. Returns a default
     * [policy][google.cloud.binaryauthorization.v1.Policy] if the project does not have one.
     *
     * The async variant is {@see BinauthzManagementServiceV1Client::getPolicyAsync()}
     * .
     *
     * @example samples/V1/BinauthzManagementServiceV1Client/get_policy.php
     *
     * @param GetPolicyRequest $request     A request to house fields associated with the call.
     * @param array            $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return Policy
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function getPolicy(GetPolicyRequest $request, array $callOptions = []): Policy
    {
        return $this->startApiCall('GetPolicy', $request, $callOptions)->wait();
    }

    /**
     * Lists [attestors][google.cloud.binaryauthorization.v1.Attestor].
     * Returns INVALID_ARGUMENT if the project does not exist.
     *
     * The async variant is
     * {@see BinauthzManagementServiceV1Client::listAttestorsAsync()} .
     *
     * @example samples/V1/BinauthzManagementServiceV1Client/list_attestors.php
     *
     * @param ListAttestorsRequest $request     A request to house fields associated with the call.
     * @param array                $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return PagedListResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function listAttestors(ListAttestorsRequest $request, array $callOptions = []): PagedListResponse
    {
        return $this->startApiCall('ListAttestors', $request, $callOptions);
    }

    /**
     * Updates an [attestor][google.cloud.binaryauthorization.v1.Attestor].
     * Returns NOT_FOUND if the [attestor][google.cloud.binaryauthorization.v1.Attestor] does not exist.
     *
     * The async variant is
     * {@see BinauthzManagementServiceV1Client::updateAttestorAsync()} .
     *
     * @example samples/V1/BinauthzManagementServiceV1Client/update_attestor.php
     *
     * @param UpdateAttestorRequest $request     A request to house fields associated with the call.
     * @param array                 $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return Attestor
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function updateAttestor(UpdateAttestorRequest $request, array $callOptions = []): Attestor
    {
        return $this->startApiCall('UpdateAttestor', $request, $callOptions)->wait();
    }

    /**
     * Creates or updates a project's [policy][google.cloud.binaryauthorization.v1.Policy], and returns a copy of the
     * new [policy][google.cloud.binaryauthorization.v1.Policy]. A policy is always updated as a whole, to avoid race
     * conditions with concurrent policy enforcement (or management!)
     * requests. Returns NOT_FOUND if the project does not exist, INVALID_ARGUMENT
     * if the request is malformed.
     *
     * The async variant is
     * {@see BinauthzManagementServiceV1Client::updatePolicyAsync()} .
     *
     * @example samples/V1/BinauthzManagementServiceV1Client/update_policy.php
     *
     * @param UpdatePolicyRequest $request     A request to house fields associated with the call.
     * @param array               $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return Policy
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function updatePolicy(UpdatePolicyRequest $request, array $callOptions = []): Policy
    {
        return $this->startApiCall('UpdatePolicy', $request, $callOptions)->wait();
    }
}
