<?php
/*
 * Copyright 2024 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * GENERATED CODE WARNING
 * Generated by gapic-generator-php from the file
 * https://github.com/googleapis/googleapis/blob/master/google/cloud/gkemulticloud/v1/aws_service.proto
 * Updates to the above are reflected here through a refresh process.
 */

namespace Google\Cloud\GkeMultiCloud\V1\Client;

use Google\ApiCore\ApiException;
use Google\ApiCore\CredentialsWrapper;
use Google\ApiCore\GapicClientTrait;
use Google\ApiCore\OperationResponse;
use Google\ApiCore\PagedListResponse;
use Google\ApiCore\ResourceHelperTrait;
use Google\ApiCore\RetrySettings;
use Google\ApiCore\Transport\TransportInterface;
use Google\ApiCore\ValidationException;
use Google\Auth\FetchAuthTokenInterface;
use Google\Cloud\GkeMultiCloud\V1\AwsCluster;
use Google\Cloud\GkeMultiCloud\V1\AwsJsonWebKeys;
use Google\Cloud\GkeMultiCloud\V1\AwsNodePool;
use Google\Cloud\GkeMultiCloud\V1\AwsOpenIdConfig;
use Google\Cloud\GkeMultiCloud\V1\AwsServerConfig;
use Google\Cloud\GkeMultiCloud\V1\CreateAwsClusterRequest;
use Google\Cloud\GkeMultiCloud\V1\CreateAwsNodePoolRequest;
use Google\Cloud\GkeMultiCloud\V1\DeleteAwsClusterRequest;
use Google\Cloud\GkeMultiCloud\V1\DeleteAwsNodePoolRequest;
use Google\Cloud\GkeMultiCloud\V1\GenerateAwsAccessTokenRequest;
use Google\Cloud\GkeMultiCloud\V1\GenerateAwsAccessTokenResponse;
use Google\Cloud\GkeMultiCloud\V1\GenerateAwsClusterAgentTokenRequest;
use Google\Cloud\GkeMultiCloud\V1\GenerateAwsClusterAgentTokenResponse;
use Google\Cloud\GkeMultiCloud\V1\GetAwsClusterRequest;
use Google\Cloud\GkeMultiCloud\V1\GetAwsJsonWebKeysRequest;
use Google\Cloud\GkeMultiCloud\V1\GetAwsNodePoolRequest;
use Google\Cloud\GkeMultiCloud\V1\GetAwsOpenIdConfigRequest;
use Google\Cloud\GkeMultiCloud\V1\GetAwsServerConfigRequest;
use Google\Cloud\GkeMultiCloud\V1\ListAwsClustersRequest;
use Google\Cloud\GkeMultiCloud\V1\ListAwsNodePoolsRequest;
use Google\Cloud\GkeMultiCloud\V1\RollbackAwsNodePoolUpdateRequest;
use Google\Cloud\GkeMultiCloud\V1\UpdateAwsClusterRequest;
use Google\Cloud\GkeMultiCloud\V1\UpdateAwsNodePoolRequest;
use Google\LongRunning\Client\OperationsClient;
use Google\LongRunning\Operation;
use GuzzleHttp\Promise\PromiseInterface;

/**
 * Service Description: The AwsClusters API provides a single centrally managed service
 * to create and manage Anthos clusters that run on AWS infrastructure.
 *
 * This class provides the ability to make remote calls to the backing service through method
 * calls that map to API methods.
 *
 * Many parameters require resource names to be formatted in a particular way. To
 * assist with these names, this class includes a format method for each type of
 * name, and additionally a parseName method to extract the individual identifiers
 * contained within formatted names that are returned by the API.
 *
 * @method PromiseInterface createAwsClusterAsync(CreateAwsClusterRequest $request, array $optionalArgs = [])
 * @method PromiseInterface createAwsNodePoolAsync(CreateAwsNodePoolRequest $request, array $optionalArgs = [])
 * @method PromiseInterface deleteAwsClusterAsync(DeleteAwsClusterRequest $request, array $optionalArgs = [])
 * @method PromiseInterface deleteAwsNodePoolAsync(DeleteAwsNodePoolRequest $request, array $optionalArgs = [])
 * @method PromiseInterface generateAwsAccessTokenAsync(GenerateAwsAccessTokenRequest $request, array $optionalArgs = [])
 * @method PromiseInterface generateAwsClusterAgentTokenAsync(GenerateAwsClusterAgentTokenRequest $request, array $optionalArgs = [])
 * @method PromiseInterface getAwsClusterAsync(GetAwsClusterRequest $request, array $optionalArgs = [])
 * @method PromiseInterface getAwsJsonWebKeysAsync(GetAwsJsonWebKeysRequest $request, array $optionalArgs = [])
 * @method PromiseInterface getAwsNodePoolAsync(GetAwsNodePoolRequest $request, array $optionalArgs = [])
 * @method PromiseInterface getAwsOpenIdConfigAsync(GetAwsOpenIdConfigRequest $request, array $optionalArgs = [])
 * @method PromiseInterface getAwsServerConfigAsync(GetAwsServerConfigRequest $request, array $optionalArgs = [])
 * @method PromiseInterface listAwsClustersAsync(ListAwsClustersRequest $request, array $optionalArgs = [])
 * @method PromiseInterface listAwsNodePoolsAsync(ListAwsNodePoolsRequest $request, array $optionalArgs = [])
 * @method PromiseInterface rollbackAwsNodePoolUpdateAsync(RollbackAwsNodePoolUpdateRequest $request, array $optionalArgs = [])
 * @method PromiseInterface updateAwsClusterAsync(UpdateAwsClusterRequest $request, array $optionalArgs = [])
 * @method PromiseInterface updateAwsNodePoolAsync(UpdateAwsNodePoolRequest $request, array $optionalArgs = [])
 */
final class AwsClustersClient
{
    use GapicClientTrait;
    use ResourceHelperTrait;

    /** The name of the service. */
    private const SERVICE_NAME = 'google.cloud.gkemulticloud.v1.AwsClusters';

    /**
     * The default address of the service.
     *
     * @deprecated SERVICE_ADDRESS_TEMPLATE should be used instead.
     */
    private const SERVICE_ADDRESS = 'gkemulticloud.googleapis.com';

    /** The address template of the service. */
    private const SERVICE_ADDRESS_TEMPLATE = 'gkemulticloud.UNIVERSE_DOMAIN';

    /** The default port of the service. */
    private const DEFAULT_SERVICE_PORT = 443;

    /** The name of the code generator, to be included in the agent header. */
    private const CODEGEN_NAME = 'gapic';

    /** The default scopes required by the service. */
    public static $serviceScopes = ['https://www.googleapis.com/auth/cloud-platform'];

    private $operationsClient;

    private static function getClientDefaults()
    {
        return [
            'serviceName' => self::SERVICE_NAME,
            'apiEndpoint' => self::SERVICE_ADDRESS . ':' . self::DEFAULT_SERVICE_PORT,
            'clientConfig' => __DIR__ . '/../resources/aws_clusters_client_config.json',
            'descriptorsConfigPath' => __DIR__ . '/../resources/aws_clusters_descriptor_config.php',
            'gcpApiConfigPath' => __DIR__ . '/../resources/aws_clusters_grpc_config.json',
            'credentialsConfig' => [
                'defaultScopes' => self::$serviceScopes,
            ],
            'transportConfig' => [
                'rest' => [
                    'restClientConfigPath' => __DIR__ . '/../resources/aws_clusters_rest_client_config.php',
                ],
            ],
        ];
    }

    /**
     * Return an OperationsClient object with the same endpoint as $this.
     *
     * @return OperationsClient
     */
    public function getOperationsClient()
    {
        return $this->operationsClient;
    }

    /**
     * Resume an existing long running operation that was previously started by a long
     * running API method. If $methodName is not provided, or does not match a long
     * running API method, then the operation can still be resumed, but the
     * OperationResponse object will not deserialize the final response.
     *
     * @param string $operationName The name of the long running operation
     * @param string $methodName    The name of the method used to start the operation
     *
     * @return OperationResponse
     */
    public function resumeOperation($operationName, $methodName = null)
    {
        $options = isset($this->descriptors[$methodName]['longRunning'])
            ? $this->descriptors[$methodName]['longRunning']
            : [];
        $operation = new OperationResponse($operationName, $this->getOperationsClient(), $options);
        $operation->reload();
        return $operation;
    }

    /**
     * Create the default operation client for the service.
     *
     * @param array $options ClientOptions for the client.
     *
     * @return OperationsClient
     */
    private function createOperationsClient(array $options)
    {
        // Unset client-specific configuration options
        unset($options['serviceName'], $options['clientConfig'], $options['descriptorsConfigPath']);

        if (isset($options['operationsClient'])) {
            return $options['operationsClient'];
        }

        return new OperationsClient($options);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a aws_cluster
     * resource.
     *
     * @param string $project
     * @param string $location
     * @param string $awsCluster
     *
     * @return string The formatted aws_cluster resource.
     */
    public static function awsClusterName(string $project, string $location, string $awsCluster): string
    {
        return self::getPathTemplate('awsCluster')->render([
            'project' => $project,
            'location' => $location,
            'aws_cluster' => $awsCluster,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * aws_node_pool resource.
     *
     * @param string $project
     * @param string $location
     * @param string $awsCluster
     * @param string $awsNodePool
     *
     * @return string The formatted aws_node_pool resource.
     */
    public static function awsNodePoolName(
        string $project,
        string $location,
        string $awsCluster,
        string $awsNodePool
    ): string {
        return self::getPathTemplate('awsNodePool')->render([
            'project' => $project,
            'location' => $location,
            'aws_cluster' => $awsCluster,
            'aws_node_pool' => $awsNodePool,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * aws_server_config resource.
     *
     * @param string $project
     * @param string $location
     *
     * @return string The formatted aws_server_config resource.
     */
    public static function awsServerConfigName(string $project, string $location): string
    {
        return self::getPathTemplate('awsServerConfig')->render([
            'project' => $project,
            'location' => $location,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a location
     * resource.
     *
     * @param string $project
     * @param string $location
     *
     * @return string The formatted location resource.
     */
    public static function locationName(string $project, string $location): string
    {
        return self::getPathTemplate('location')->render([
            'project' => $project,
            'location' => $location,
        ]);
    }

    /**
     * Parses a formatted name string and returns an associative array of the components in the name.
     * The following name formats are supported:
     * Template: Pattern
     * - awsCluster: projects/{project}/locations/{location}/awsClusters/{aws_cluster}
     * - awsNodePool: projects/{project}/locations/{location}/awsClusters/{aws_cluster}/awsNodePools/{aws_node_pool}
     * - awsServerConfig: projects/{project}/locations/{location}/awsServerConfig
     * - location: projects/{project}/locations/{location}
     *
     * The optional $template argument can be supplied to specify a particular pattern,
     * and must match one of the templates listed above. If no $template argument is
     * provided, or if the $template argument does not match one of the templates
     * listed, then parseName will check each of the supported templates, and return
     * the first match.
     *
     * @param string $formattedName The formatted name string
     * @param string $template      Optional name of template to match
     *
     * @return array An associative array from name component IDs to component values.
     *
     * @throws ValidationException If $formattedName could not be matched.
     */
    public static function parseName(string $formattedName, string $template = null): array
    {
        return self::parseFormattedName($formattedName, $template);
    }

    /**
     * Constructor.
     *
     * @param array $options {
     *     Optional. Options for configuring the service API wrapper.
     *
     *     @type string $apiEndpoint
     *           The address of the API remote host. May optionally include the port, formatted
     *           as "<uri>:<port>". Default 'gkemulticloud.googleapis.com:443'.
     *     @type string|array|FetchAuthTokenInterface|CredentialsWrapper $credentials
     *           The credentials to be used by the client to authorize API calls. This option
     *           accepts either a path to a credentials file, or a decoded credentials file as a
     *           PHP array.
     *           *Advanced usage*: In addition, this option can also accept a pre-constructed
     *           {@see \Google\Auth\FetchAuthTokenInterface} object or
     *           {@see \Google\ApiCore\CredentialsWrapper} object. Note that when one of these
     *           objects are provided, any settings in $credentialsConfig will be ignored.
     *     @type array $credentialsConfig
     *           Options used to configure credentials, including auth token caching, for the
     *           client. For a full list of supporting configuration options, see
     *           {@see \Google\ApiCore\CredentialsWrapper::build()} .
     *     @type bool $disableRetries
     *           Determines whether or not retries defined by the client configuration should be
     *           disabled. Defaults to `false`.
     *     @type string|array $clientConfig
     *           Client method configuration, including retry settings. This option can be either
     *           a path to a JSON file, or a PHP array containing the decoded JSON data. By
     *           default this settings points to the default client config file, which is
     *           provided in the resources folder.
     *     @type string|TransportInterface $transport
     *           The transport used for executing network requests. May be either the string
     *           `rest` or `grpc`. Defaults to `grpc` if gRPC support is detected on the system.
     *           *Advanced usage*: Additionally, it is possible to pass in an already
     *           instantiated {@see \Google\ApiCore\Transport\TransportInterface} object. Note
     *           that when this object is provided, any settings in $transportConfig, and any
     *           $apiEndpoint setting, will be ignored.
     *     @type array $transportConfig
     *           Configuration options that will be used to construct the transport. Options for
     *           each supported transport type should be passed in a key for that transport. For
     *           example:
     *           $transportConfig = [
     *               'grpc' => [...],
     *               'rest' => [...],
     *           ];
     *           See the {@see \Google\ApiCore\Transport\GrpcTransport::build()} and
     *           {@see \Google\ApiCore\Transport\RestTransport::build()} methods for the
     *           supported options.
     *     @type callable $clientCertSource
     *           A callable which returns the client cert as a string. This can be used to
     *           provide a certificate and private key to the transport layer for mTLS.
     * }
     *
     * @throws ValidationException
     */
    public function __construct(array $options = [])
    {
        $clientOptions = $this->buildClientOptions($options);
        $this->setClientOptions($clientOptions);
        $this->operationsClient = $this->createOperationsClient($clientOptions);
    }

    /** Handles execution of the async variants for each documented method. */
    public function __call($method, $args)
    {
        if (substr($method, -5) !== 'Async') {
            trigger_error('Call to undefined method ' . __CLASS__ . "::$method()", E_USER_ERROR);
        }

        array_unshift($args, substr($method, 0, -5));
        return call_user_func_array([$this, 'startAsyncCall'], $args);
    }

    /**
     * Creates a new [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster]
     * resource on a given Google Cloud Platform project and region.
     *
     * If successful, the response contains a newly created
     * [Operation][google.longrunning.Operation] resource that can be
     * described to track the status of the operation.
     *
     * The async variant is {@see AwsClustersClient::createAwsClusterAsync()} .
     *
     * @example samples/V1/AwsClustersClient/create_aws_cluster.php
     *
     * @param CreateAwsClusterRequest $request     A request to house fields associated with the call.
     * @param array                   $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return OperationResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function createAwsCluster(CreateAwsClusterRequest $request, array $callOptions = []): OperationResponse
    {
        return $this->startApiCall('CreateAwsCluster', $request, $callOptions)->wait();
    }

    /**
     * Creates a new [AwsNodePool][google.cloud.gkemulticloud.v1.AwsNodePool],
     * attached to a given [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster].
     *
     * If successful, the response contains a newly created
     * [Operation][google.longrunning.Operation] resource that can be
     * described to track the status of the operation.
     *
     * The async variant is {@see AwsClustersClient::createAwsNodePoolAsync()} .
     *
     * @example samples/V1/AwsClustersClient/create_aws_node_pool.php
     *
     * @param CreateAwsNodePoolRequest $request     A request to house fields associated with the call.
     * @param array                    $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return OperationResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function createAwsNodePool(CreateAwsNodePoolRequest $request, array $callOptions = []): OperationResponse
    {
        return $this->startApiCall('CreateAwsNodePool', $request, $callOptions)->wait();
    }

    /**
     * Deletes a specific [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster]
     * resource.
     *
     * Fails if the cluster has one or more associated
     * [AwsNodePool][google.cloud.gkemulticloud.v1.AwsNodePool] resources.
     *
     * If successful, the response contains a newly created
     * [Operation][google.longrunning.Operation] resource that can be
     * described to track the status of the operation.
     *
     * The async variant is {@see AwsClustersClient::deleteAwsClusterAsync()} .
     *
     * @example samples/V1/AwsClustersClient/delete_aws_cluster.php
     *
     * @param DeleteAwsClusterRequest $request     A request to house fields associated with the call.
     * @param array                   $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return OperationResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function deleteAwsCluster(DeleteAwsClusterRequest $request, array $callOptions = []): OperationResponse
    {
        return $this->startApiCall('DeleteAwsCluster', $request, $callOptions)->wait();
    }

    /**
     * Deletes a specific [AwsNodePool][google.cloud.gkemulticloud.v1.AwsNodePool]
     * resource.
     *
     * If successful, the response contains a newly created
     * [Operation][google.longrunning.Operation] resource that can be
     * described to track the status of the operation.
     *
     * The async variant is {@see AwsClustersClient::deleteAwsNodePoolAsync()} .
     *
     * @example samples/V1/AwsClustersClient/delete_aws_node_pool.php
     *
     * @param DeleteAwsNodePoolRequest $request     A request to house fields associated with the call.
     * @param array                    $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return OperationResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function deleteAwsNodePool(DeleteAwsNodePoolRequest $request, array $callOptions = []): OperationResponse
    {
        return $this->startApiCall('DeleteAwsNodePool', $request, $callOptions)->wait();
    }

    /**
     * Generates a short-lived access token to authenticate to a given
     * [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster] resource.
     *
     * The async variant is {@see AwsClustersClient::generateAwsAccessTokenAsync()} .
     *
     * @example samples/V1/AwsClustersClient/generate_aws_access_token.php
     *
     * @param GenerateAwsAccessTokenRequest $request     A request to house fields associated with the call.
     * @param array                         $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return GenerateAwsAccessTokenResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function generateAwsAccessToken(
        GenerateAwsAccessTokenRequest $request,
        array $callOptions = []
    ): GenerateAwsAccessTokenResponse {
        return $this->startApiCall('GenerateAwsAccessToken', $request, $callOptions)->wait();
    }

    /**
     * Generates an access token for a cluster agent.
     *
     * The async variant is
     * {@see AwsClustersClient::generateAwsClusterAgentTokenAsync()} .
     *
     * @example samples/V1/AwsClustersClient/generate_aws_cluster_agent_token.php
     *
     * @param GenerateAwsClusterAgentTokenRequest $request     A request to house fields associated with the call.
     * @param array                               $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return GenerateAwsClusterAgentTokenResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function generateAwsClusterAgentToken(
        GenerateAwsClusterAgentTokenRequest $request,
        array $callOptions = []
    ): GenerateAwsClusterAgentTokenResponse {
        return $this->startApiCall('GenerateAwsClusterAgentToken', $request, $callOptions)->wait();
    }

    /**
     * Describes a specific [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster]
     * resource.
     *
     * The async variant is {@see AwsClustersClient::getAwsClusterAsync()} .
     *
     * @example samples/V1/AwsClustersClient/get_aws_cluster.php
     *
     * @param GetAwsClusterRequest $request     A request to house fields associated with the call.
     * @param array                $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return AwsCluster
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function getAwsCluster(GetAwsClusterRequest $request, array $callOptions = []): AwsCluster
    {
        return $this->startApiCall('GetAwsCluster', $request, $callOptions)->wait();
    }

    /**
     * Gets the public component of the cluster signing keys in
     * JSON Web Key format.
     *
     * The async variant is {@see AwsClustersClient::getAwsJsonWebKeysAsync()} .
     *
     * @example samples/V1/AwsClustersClient/get_aws_json_web_keys.php
     *
     * @param GetAwsJsonWebKeysRequest $request     A request to house fields associated with the call.
     * @param array                    $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return AwsJsonWebKeys
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function getAwsJsonWebKeys(GetAwsJsonWebKeysRequest $request, array $callOptions = []): AwsJsonWebKeys
    {
        return $this->startApiCall('GetAwsJsonWebKeys', $request, $callOptions)->wait();
    }

    /**
     * Describes a specific
     * [AwsNodePool][google.cloud.gkemulticloud.v1.AwsNodePool] resource.
     *
     * The async variant is {@see AwsClustersClient::getAwsNodePoolAsync()} .
     *
     * @example samples/V1/AwsClustersClient/get_aws_node_pool.php
     *
     * @param GetAwsNodePoolRequest $request     A request to house fields associated with the call.
     * @param array                 $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return AwsNodePool
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function getAwsNodePool(GetAwsNodePoolRequest $request, array $callOptions = []): AwsNodePool
    {
        return $this->startApiCall('GetAwsNodePool', $request, $callOptions)->wait();
    }

    /**
     * Gets the OIDC discovery document for the cluster.
     * See the
     * [OpenID Connect Discovery 1.0
     * specification](https://openid.net/specs/openid-connect-discovery-1_0.html)
     * for details.
     *
     * The async variant is {@see AwsClustersClient::getAwsOpenIdConfigAsync()} .
     *
     * @example samples/V1/AwsClustersClient/get_aws_open_id_config.php
     *
     * @param GetAwsOpenIdConfigRequest $request     A request to house fields associated with the call.
     * @param array                     $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return AwsOpenIdConfig
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function getAwsOpenIdConfig(GetAwsOpenIdConfigRequest $request, array $callOptions = []): AwsOpenIdConfig
    {
        return $this->startApiCall('GetAwsOpenIdConfig', $request, $callOptions)->wait();
    }

    /**
     * Returns information, such as supported AWS regions and Kubernetes
     * versions, on a given Google Cloud location.
     *
     * The async variant is {@see AwsClustersClient::getAwsServerConfigAsync()} .
     *
     * @example samples/V1/AwsClustersClient/get_aws_server_config.php
     *
     * @param GetAwsServerConfigRequest $request     A request to house fields associated with the call.
     * @param array                     $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return AwsServerConfig
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function getAwsServerConfig(GetAwsServerConfigRequest $request, array $callOptions = []): AwsServerConfig
    {
        return $this->startApiCall('GetAwsServerConfig', $request, $callOptions)->wait();
    }

    /**
     * Lists all [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster] resources
     * on a given Google Cloud project and region.
     *
     * The async variant is {@see AwsClustersClient::listAwsClustersAsync()} .
     *
     * @example samples/V1/AwsClustersClient/list_aws_clusters.php
     *
     * @param ListAwsClustersRequest $request     A request to house fields associated with the call.
     * @param array                  $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return PagedListResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function listAwsClusters(ListAwsClustersRequest $request, array $callOptions = []): PagedListResponse
    {
        return $this->startApiCall('ListAwsClusters', $request, $callOptions);
    }

    /**
     * Lists all [AwsNodePool][google.cloud.gkemulticloud.v1.AwsNodePool]
     * resources on a given
     * [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster].
     *
     * The async variant is {@see AwsClustersClient::listAwsNodePoolsAsync()} .
     *
     * @example samples/V1/AwsClustersClient/list_aws_node_pools.php
     *
     * @param ListAwsNodePoolsRequest $request     A request to house fields associated with the call.
     * @param array                   $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return PagedListResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function listAwsNodePools(ListAwsNodePoolsRequest $request, array $callOptions = []): PagedListResponse
    {
        return $this->startApiCall('ListAwsNodePools', $request, $callOptions);
    }

    /**
     * Rolls back a previously aborted or failed
     * [AwsNodePool][google.cloud.gkemulticloud.v1.AwsNodePool] update request.
     * Makes no changes if the last update request successfully finished.
     * If an update request is in progress, you cannot rollback the update.
     * You must first cancel or let it finish unsuccessfully before you can
     * rollback.
     *
     * The async variant is {@see AwsClustersClient::rollbackAwsNodePoolUpdateAsync()}
     * .
     *
     * @example samples/V1/AwsClustersClient/rollback_aws_node_pool_update.php
     *
     * @param RollbackAwsNodePoolUpdateRequest $request     A request to house fields associated with the call.
     * @param array                            $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return OperationResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function rollbackAwsNodePoolUpdate(
        RollbackAwsNodePoolUpdateRequest $request,
        array $callOptions = []
    ): OperationResponse {
        return $this->startApiCall('RollbackAwsNodePoolUpdate', $request, $callOptions)->wait();
    }

    /**
     * Updates an [AwsCluster][google.cloud.gkemulticloud.v1.AwsCluster].
     *
     * The async variant is {@see AwsClustersClient::updateAwsClusterAsync()} .
     *
     * @example samples/V1/AwsClustersClient/update_aws_cluster.php
     *
     * @param UpdateAwsClusterRequest $request     A request to house fields associated with the call.
     * @param array                   $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return OperationResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function updateAwsCluster(UpdateAwsClusterRequest $request, array $callOptions = []): OperationResponse
    {
        return $this->startApiCall('UpdateAwsCluster', $request, $callOptions)->wait();
    }

    /**
     * Updates an [AwsNodePool][google.cloud.gkemulticloud.v1.AwsNodePool].
     *
     * The async variant is {@see AwsClustersClient::updateAwsNodePoolAsync()} .
     *
     * @example samples/V1/AwsClustersClient/update_aws_node_pool.php
     *
     * @param UpdateAwsNodePoolRequest $request     A request to house fields associated with the call.
     * @param array                    $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return OperationResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function updateAwsNodePool(UpdateAwsNodePoolRequest $request, array $callOptions = []): OperationResponse
    {
        return $this->startApiCall('UpdateAwsNodePool', $request, $callOptions)->wait();
    }
}
