<?php
/*
 * Copyright 2023 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * GENERATED CODE WARNING
 * Generated by gapic-generator-php from the file
 * https://github.com/googleapis/googleapis/blob/master/google/monitoring/v3/alert_service.proto
 * Updates to the above are reflected here through a refresh process.
 */

namespace Google\Cloud\Monitoring\V3\Client;

use Google\ApiCore\ApiException;
use Google\ApiCore\CredentialsWrapper;
use Google\ApiCore\GapicClientTrait;
use Google\ApiCore\PagedListResponse;
use Google\ApiCore\ResourceHelperTrait;
use Google\ApiCore\RetrySettings;
use Google\ApiCore\Transport\TransportInterface;
use Google\ApiCore\ValidationException;
use Google\Auth\FetchAuthTokenInterface;
use Google\Cloud\Monitoring\V3\AlertPolicy;
use Google\Cloud\Monitoring\V3\CreateAlertPolicyRequest;
use Google\Cloud\Monitoring\V3\DeleteAlertPolicyRequest;
use Google\Cloud\Monitoring\V3\GetAlertPolicyRequest;
use Google\Cloud\Monitoring\V3\ListAlertPoliciesRequest;
use Google\Cloud\Monitoring\V3\UpdateAlertPolicyRequest;
use GuzzleHttp\Promise\PromiseInterface;

/**
 * Service Description: The AlertPolicyService API is used to manage (list, create, delete,
 * edit) alert policies in Cloud Monitoring. An alerting policy is
 * a description of the conditions under which some aspect of your
 * system is considered to be "unhealthy" and the ways to notify
 * people or services about this state. In addition to using this API, alert
 * policies can also be managed through
 * [Cloud Monitoring](https://cloud.google.com/monitoring/docs/),
 * which can be reached by clicking the "Monitoring" tab in
 * [Cloud console](https://console.cloud.google.com/).
 *
 * This class provides the ability to make remote calls to the backing service through method
 * calls that map to API methods.
 *
 * Many parameters require resource names to be formatted in a particular way. To
 * assist with these names, this class includes a format method for each type of
 * name, and additionally a parseName method to extract the individual identifiers
 * contained within formatted names that are returned by the API.
 *
 * @method PromiseInterface createAlertPolicyAsync(CreateAlertPolicyRequest $request, array $optionalArgs = [])
 * @method PromiseInterface deleteAlertPolicyAsync(DeleteAlertPolicyRequest $request, array $optionalArgs = [])
 * @method PromiseInterface getAlertPolicyAsync(GetAlertPolicyRequest $request, array $optionalArgs = [])
 * @method PromiseInterface listAlertPoliciesAsync(ListAlertPoliciesRequest $request, array $optionalArgs = [])
 * @method PromiseInterface updateAlertPolicyAsync(UpdateAlertPolicyRequest $request, array $optionalArgs = [])
 */
final class AlertPolicyServiceClient
{
    use GapicClientTrait;
    use ResourceHelperTrait;

    /** The name of the service. */
    private const SERVICE_NAME = 'google.monitoring.v3.AlertPolicyService';

    /**
     * The default address of the service.
     *
     * @deprecated SERVICE_ADDRESS_TEMPLATE should be used instead.
     */
    private const SERVICE_ADDRESS = 'monitoring.googleapis.com';

    /** The address template of the service. */
    private const SERVICE_ADDRESS_TEMPLATE = 'monitoring.UNIVERSE_DOMAIN';

    /** The default port of the service. */
    private const DEFAULT_SERVICE_PORT = 443;

    /** The name of the code generator, to be included in the agent header. */
    private const CODEGEN_NAME = 'gapic';

    /** The default scopes required by the service. */
    public static $serviceScopes = [
        'https://www.googleapis.com/auth/cloud-platform',
        'https://www.googleapis.com/auth/monitoring',
        'https://www.googleapis.com/auth/monitoring.read',
    ];

    private static function getClientDefaults()
    {
        return [
            'serviceName' => self::SERVICE_NAME,
            'apiEndpoint' => self::SERVICE_ADDRESS . ':' . self::DEFAULT_SERVICE_PORT,
            'clientConfig' => __DIR__ . '/../resources/alert_policy_service_client_config.json',
            'descriptorsConfigPath' => __DIR__ . '/../resources/alert_policy_service_descriptor_config.php',
            'gcpApiConfigPath' => __DIR__ . '/../resources/alert_policy_service_grpc_config.json',
            'credentialsConfig' => [
                'defaultScopes' => self::$serviceScopes,
            ],
            'transportConfig' => [
                'rest' => [
                    'restClientConfigPath' => __DIR__ . '/../resources/alert_policy_service_rest_client_config.php',
                ],
            ],
        ];
    }

    /**
     * Formats a string containing the fully-qualified path to represent a alert_policy
     * resource.
     *
     * @param string $project
     * @param string $alertPolicy
     *
     * @return string The formatted alert_policy resource.
     */
    public static function alertPolicyName(string $project, string $alertPolicy): string
    {
        return self::getPathTemplate('alertPolicy')->render([
            'project' => $project,
            'alert_policy' => $alertPolicy,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * alert_policy_condition resource.
     *
     * @param string $project
     * @param string $alertPolicy
     * @param string $condition
     *
     * @return string The formatted alert_policy_condition resource.
     */
    public static function alertPolicyConditionName(string $project, string $alertPolicy, string $condition): string
    {
        return self::getPathTemplate('alertPolicyCondition')->render([
            'project' => $project,
            'alert_policy' => $alertPolicy,
            'condition' => $condition,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * folder_alert_policy resource.
     *
     * @param string $folder
     * @param string $alertPolicy
     *
     * @return string The formatted folder_alert_policy resource.
     */
    public static function folderAlertPolicyName(string $folder, string $alertPolicy): string
    {
        return self::getPathTemplate('folderAlertPolicy')->render([
            'folder' => $folder,
            'alert_policy' => $alertPolicy,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * folder_alert_policy_condition resource.
     *
     * @param string $folder
     * @param string $alertPolicy
     * @param string $condition
     *
     * @return string The formatted folder_alert_policy_condition resource.
     */
    public static function folderAlertPolicyConditionName(string $folder, string $alertPolicy, string $condition): string
    {
        return self::getPathTemplate('folderAlertPolicyCondition')->render([
            'folder' => $folder,
            'alert_policy' => $alertPolicy,
            'condition' => $condition,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * organization_alert_policy resource.
     *
     * @param string $organization
     * @param string $alertPolicy
     *
     * @return string The formatted organization_alert_policy resource.
     */
    public static function organizationAlertPolicyName(string $organization, string $alertPolicy): string
    {
        return self::getPathTemplate('organizationAlertPolicy')->render([
            'organization' => $organization,
            'alert_policy' => $alertPolicy,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * organization_alert_policy_condition resource.
     *
     * @param string $organization
     * @param string $alertPolicy
     * @param string $condition
     *
     * @return string The formatted organization_alert_policy_condition resource.
     */
    public static function organizationAlertPolicyConditionName(string $organization, string $alertPolicy, string $condition): string
    {
        return self::getPathTemplate('organizationAlertPolicyCondition')->render([
            'organization' => $organization,
            'alert_policy' => $alertPolicy,
            'condition' => $condition,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * project_alert_policy resource.
     *
     * @param string $project
     * @param string $alertPolicy
     *
     * @return string The formatted project_alert_policy resource.
     */
    public static function projectAlertPolicyName(string $project, string $alertPolicy): string
    {
        return self::getPathTemplate('projectAlertPolicy')->render([
            'project' => $project,
            'alert_policy' => $alertPolicy,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * project_alert_policy_condition resource.
     *
     * @param string $project
     * @param string $alertPolicy
     * @param string $condition
     *
     * @return string The formatted project_alert_policy_condition resource.
     */
    public static function projectAlertPolicyConditionName(string $project, string $alertPolicy, string $condition): string
    {
        return self::getPathTemplate('projectAlertPolicyCondition')->render([
            'project' => $project,
            'alert_policy' => $alertPolicy,
            'condition' => $condition,
        ]);
    }

    /**
     * Parses a formatted name string and returns an associative array of the components in the name.
     * The following name formats are supported:
     * Template: Pattern
     * - alertPolicy: projects/{project}/alertPolicies/{alert_policy}
     * - alertPolicyCondition: projects/{project}/alertPolicies/{alert_policy}/conditions/{condition}
     * - folderAlertPolicy: folders/{folder}/alertPolicies/{alert_policy}
     * - folderAlertPolicyCondition: folders/{folder}/alertPolicies/{alert_policy}/conditions/{condition}
     * - organizationAlertPolicy: organizations/{organization}/alertPolicies/{alert_policy}
     * - organizationAlertPolicyCondition: organizations/{organization}/alertPolicies/{alert_policy}/conditions/{condition}
     * - projectAlertPolicy: projects/{project}/alertPolicies/{alert_policy}
     * - projectAlertPolicyCondition: projects/{project}/alertPolicies/{alert_policy}/conditions/{condition}
     *
     * The optional $template argument can be supplied to specify a particular pattern,
     * and must match one of the templates listed above. If no $template argument is
     * provided, or if the $template argument does not match one of the templates
     * listed, then parseName will check each of the supported templates, and return
     * the first match.
     *
     * @param string $formattedName The formatted name string
     * @param string $template      Optional name of template to match
     *
     * @return array An associative array from name component IDs to component values.
     *
     * @throws ValidationException If $formattedName could not be matched.
     */
    public static function parseName(string $formattedName, string $template = null): array
    {
        return self::parseFormattedName($formattedName, $template);
    }

    /**
     * Constructor.
     *
     * @param array $options {
     *     Optional. Options for configuring the service API wrapper.
     *
     *     @type string $apiEndpoint
     *           The address of the API remote host. May optionally include the port, formatted
     *           as "<uri>:<port>". Default 'monitoring.googleapis.com:443'.
     *     @type string|array|FetchAuthTokenInterface|CredentialsWrapper $credentials
     *           The credentials to be used by the client to authorize API calls. This option
     *           accepts either a path to a credentials file, or a decoded credentials file as a
     *           PHP array.
     *           *Advanced usage*: In addition, this option can also accept a pre-constructed
     *           {@see \Google\Auth\FetchAuthTokenInterface} object or
     *           {@see \Google\ApiCore\CredentialsWrapper} object. Note that when one of these
     *           objects are provided, any settings in $credentialsConfig will be ignored.
     *     @type array $credentialsConfig
     *           Options used to configure credentials, including auth token caching, for the
     *           client. For a full list of supporting configuration options, see
     *           {@see \Google\ApiCore\CredentialsWrapper::build()} .
     *     @type bool $disableRetries
     *           Determines whether or not retries defined by the client configuration should be
     *           disabled. Defaults to `false`.
     *     @type string|array $clientConfig
     *           Client method configuration, including retry settings. This option can be either
     *           a path to a JSON file, or a PHP array containing the decoded JSON data. By
     *           default this settings points to the default client config file, which is
     *           provided in the resources folder.
     *     @type string|TransportInterface $transport
     *           The transport used for executing network requests. May be either the string
     *           `rest` or `grpc`. Defaults to `grpc` if gRPC support is detected on the system.
     *           *Advanced usage*: Additionally, it is possible to pass in an already
     *           instantiated {@see \Google\ApiCore\Transport\TransportInterface} object. Note
     *           that when this object is provided, any settings in $transportConfig, and any
     *           $apiEndpoint setting, will be ignored.
     *     @type array $transportConfig
     *           Configuration options that will be used to construct the transport. Options for
     *           each supported transport type should be passed in a key for that transport. For
     *           example:
     *           $transportConfig = [
     *               'grpc' => [...],
     *               'rest' => [...],
     *           ];
     *           See the {@see \Google\ApiCore\Transport\GrpcTransport::build()} and
     *           {@see \Google\ApiCore\Transport\RestTransport::build()} methods for the
     *           supported options.
     *     @type callable $clientCertSource
     *           A callable which returns the client cert as a string. This can be used to
     *           provide a certificate and private key to the transport layer for mTLS.
     * }
     *
     * @throws ValidationException
     */
    public function __construct(array $options = [])
    {
        $clientOptions = $this->buildClientOptions($options);
        $this->setClientOptions($clientOptions);
    }

    /** Handles execution of the async variants for each documented method. */
    public function __call($method, $args)
    {
        if (substr($method, -5) !== 'Async') {
            trigger_error('Call to undefined method ' . __CLASS__ . "::$method()", E_USER_ERROR);
        }

        array_unshift($args, substr($method, 0, -5));
        return call_user_func_array([$this, 'startAsyncCall'], $args);
    }

    /**
     * Creates a new alerting policy.
     *
     * Design your application to single-thread API calls that modify the state of
     * alerting policies in a single project. This includes calls to
     * CreateAlertPolicy, DeleteAlertPolicy and UpdateAlertPolicy.
     *
     * The async variant is {@see AlertPolicyServiceClient::createAlertPolicyAsync()} .
     *
     * @example samples/V3/AlertPolicyServiceClient/create_alert_policy.php
     *
     * @param CreateAlertPolicyRequest $request     A request to house fields associated with the call.
     * @param array                    $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return AlertPolicy
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function createAlertPolicy(CreateAlertPolicyRequest $request, array $callOptions = []): AlertPolicy
    {
        return $this->startApiCall('CreateAlertPolicy', $request, $callOptions)->wait();
    }

    /**
     * Deletes an alerting policy.
     *
     * Design your application to single-thread API calls that modify the state of
     * alerting policies in a single project. This includes calls to
     * CreateAlertPolicy, DeleteAlertPolicy and UpdateAlertPolicy.
     *
     * The async variant is {@see AlertPolicyServiceClient::deleteAlertPolicyAsync()} .
     *
     * @example samples/V3/AlertPolicyServiceClient/delete_alert_policy.php
     *
     * @param DeleteAlertPolicyRequest $request     A request to house fields associated with the call.
     * @param array                    $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function deleteAlertPolicy(DeleteAlertPolicyRequest $request, array $callOptions = []): void
    {
        $this->startApiCall('DeleteAlertPolicy', $request, $callOptions)->wait();
    }

    /**
     * Gets a single alerting policy.
     *
     * The async variant is {@see AlertPolicyServiceClient::getAlertPolicyAsync()} .
     *
     * @example samples/V3/AlertPolicyServiceClient/get_alert_policy.php
     *
     * @param GetAlertPolicyRequest $request     A request to house fields associated with the call.
     * @param array                 $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return AlertPolicy
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function getAlertPolicy(GetAlertPolicyRequest $request, array $callOptions = []): AlertPolicy
    {
        return $this->startApiCall('GetAlertPolicy', $request, $callOptions)->wait();
    }

    /**
     * Lists the existing alerting policies for the workspace.
     *
     * The async variant is {@see AlertPolicyServiceClient::listAlertPoliciesAsync()} .
     *
     * @example samples/V3/AlertPolicyServiceClient/list_alert_policies.php
     *
     * @param ListAlertPoliciesRequest $request     A request to house fields associated with the call.
     * @param array                    $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return PagedListResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function listAlertPolicies(ListAlertPoliciesRequest $request, array $callOptions = []): PagedListResponse
    {
        return $this->startApiCall('ListAlertPolicies', $request, $callOptions);
    }

    /**
     * Updates an alerting policy. You can either replace the entire policy with
     * a new one or replace only certain fields in the current alerting policy by
     * specifying the fields to be updated via `updateMask`. Returns the
     * updated alerting policy.
     *
     * Design your application to single-thread API calls that modify the state of
     * alerting policies in a single project. This includes calls to
     * CreateAlertPolicy, DeleteAlertPolicy and UpdateAlertPolicy.
     *
     * The async variant is {@see AlertPolicyServiceClient::updateAlertPolicyAsync()} .
     *
     * @example samples/V3/AlertPolicyServiceClient/update_alert_policy.php
     *
     * @param UpdateAlertPolicyRequest $request     A request to house fields associated with the call.
     * @param array                    $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return AlertPolicy
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function updateAlertPolicy(UpdateAlertPolicyRequest $request, array $callOptions = []): AlertPolicy
    {
        return $this->startApiCall('UpdateAlertPolicy', $request, $callOptions)->wait();
    }
}
