<?php
/*
 * Copyright 2024 Google LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * GENERATED CODE WARNING
 * Generated by gapic-generator-php from the file
 * https://github.com/googleapis/googleapis/blob/master/google/cloud/retail/v2/serving_config_service.proto
 * Updates to the above are reflected here through a refresh process.
 */

namespace Google\Cloud\Retail\V2\Client;

use Google\ApiCore\ApiException;
use Google\ApiCore\CredentialsWrapper;
use Google\ApiCore\GapicClientTrait;
use Google\ApiCore\PagedListResponse;
use Google\ApiCore\ResourceHelperTrait;
use Google\ApiCore\RetrySettings;
use Google\ApiCore\Transport\TransportInterface;
use Google\ApiCore\ValidationException;
use Google\Auth\FetchAuthTokenInterface;
use Google\Cloud\Retail\V2\AddControlRequest;
use Google\Cloud\Retail\V2\CreateServingConfigRequest;
use Google\Cloud\Retail\V2\DeleteServingConfigRequest;
use Google\Cloud\Retail\V2\GetServingConfigRequest;
use Google\Cloud\Retail\V2\ListServingConfigsRequest;
use Google\Cloud\Retail\V2\RemoveControlRequest;
use Google\Cloud\Retail\V2\ServingConfig;
use Google\Cloud\Retail\V2\UpdateServingConfigRequest;
use GuzzleHttp\Promise\PromiseInterface;

/**
 * Service Description: Service for modifying ServingConfig.
 *
 * This class provides the ability to make remote calls to the backing service through method
 * calls that map to API methods.
 *
 * Many parameters require resource names to be formatted in a particular way. To
 * assist with these names, this class includes a format method for each type of
 * name, and additionally a parseName method to extract the individual identifiers
 * contained within formatted names that are returned by the API.
 *
 * @method PromiseInterface addControlAsync(AddControlRequest $request, array $optionalArgs = [])
 * @method PromiseInterface createServingConfigAsync(CreateServingConfigRequest $request, array $optionalArgs = [])
 * @method PromiseInterface deleteServingConfigAsync(DeleteServingConfigRequest $request, array $optionalArgs = [])
 * @method PromiseInterface getServingConfigAsync(GetServingConfigRequest $request, array $optionalArgs = [])
 * @method PromiseInterface listServingConfigsAsync(ListServingConfigsRequest $request, array $optionalArgs = [])
 * @method PromiseInterface removeControlAsync(RemoveControlRequest $request, array $optionalArgs = [])
 * @method PromiseInterface updateServingConfigAsync(UpdateServingConfigRequest $request, array $optionalArgs = [])
 */
final class ServingConfigServiceClient
{
    use GapicClientTrait;
    use ResourceHelperTrait;

    /** The name of the service. */
    private const SERVICE_NAME = 'google.cloud.retail.v2.ServingConfigService';

    /**
     * The default address of the service.
     *
     * @deprecated SERVICE_ADDRESS_TEMPLATE should be used instead.
     */
    private const SERVICE_ADDRESS = 'retail.googleapis.com';

    /** The address template of the service. */
    private const SERVICE_ADDRESS_TEMPLATE = 'retail.UNIVERSE_DOMAIN';

    /** The default port of the service. */
    private const DEFAULT_SERVICE_PORT = 443;

    /** The name of the code generator, to be included in the agent header. */
    private const CODEGEN_NAME = 'gapic';

    /** The default scopes required by the service. */
    public static $serviceScopes = ['https://www.googleapis.com/auth/cloud-platform'];

    private static function getClientDefaults()
    {
        return [
            'serviceName' => self::SERVICE_NAME,
            'apiEndpoint' => self::SERVICE_ADDRESS . ':' . self::DEFAULT_SERVICE_PORT,
            'clientConfig' => __DIR__ . '/../resources/serving_config_service_client_config.json',
            'descriptorsConfigPath' => __DIR__ . '/../resources/serving_config_service_descriptor_config.php',
            'gcpApiConfigPath' => __DIR__ . '/../resources/serving_config_service_grpc_config.json',
            'credentialsConfig' => [
                'defaultScopes' => self::$serviceScopes,
            ],
            'transportConfig' => [
                'rest' => [
                    'restClientConfigPath' => __DIR__ . '/../resources/serving_config_service_rest_client_config.php',
                ],
            ],
        ];
    }

    /**
     * Formats a string containing the fully-qualified path to represent a catalog
     * resource.
     *
     * @param string $project
     * @param string $location
     * @param string $catalog
     *
     * @return string The formatted catalog resource.
     */
    public static function catalogName(string $project, string $location, string $catalog): string
    {
        return self::getPathTemplate('catalog')->render([
            'project' => $project,
            'location' => $location,
            'catalog' => $catalog,
        ]);
    }

    /**
     * Formats a string containing the fully-qualified path to represent a
     * serving_config resource.
     *
     * @param string $project
     * @param string $location
     * @param string $catalog
     * @param string $servingConfig
     *
     * @return string The formatted serving_config resource.
     */
    public static function servingConfigName(
        string $project,
        string $location,
        string $catalog,
        string $servingConfig
    ): string {
        return self::getPathTemplate('servingConfig')->render([
            'project' => $project,
            'location' => $location,
            'catalog' => $catalog,
            'serving_config' => $servingConfig,
        ]);
    }

    /**
     * Parses a formatted name string and returns an associative array of the components in the name.
     * The following name formats are supported:
     * Template: Pattern
     * - catalog: projects/{project}/locations/{location}/catalogs/{catalog}
     * - servingConfig: projects/{project}/locations/{location}/catalogs/{catalog}/servingConfigs/{serving_config}
     *
     * The optional $template argument can be supplied to specify a particular pattern,
     * and must match one of the templates listed above. If no $template argument is
     * provided, or if the $template argument does not match one of the templates
     * listed, then parseName will check each of the supported templates, and return
     * the first match.
     *
     * @param string $formattedName The formatted name string
     * @param string $template      Optional name of template to match
     *
     * @return array An associative array from name component IDs to component values.
     *
     * @throws ValidationException If $formattedName could not be matched.
     */
    public static function parseName(string $formattedName, string $template = null): array
    {
        return self::parseFormattedName($formattedName, $template);
    }

    /**
     * Constructor.
     *
     * @param array $options {
     *     Optional. Options for configuring the service API wrapper.
     *
     *     @type string $apiEndpoint
     *           The address of the API remote host. May optionally include the port, formatted
     *           as "<uri>:<port>". Default 'retail.googleapis.com:443'.
     *     @type string|array|FetchAuthTokenInterface|CredentialsWrapper $credentials
     *           The credentials to be used by the client to authorize API calls. This option
     *           accepts either a path to a credentials file, or a decoded credentials file as a
     *           PHP array.
     *           *Advanced usage*: In addition, this option can also accept a pre-constructed
     *           {@see \Google\Auth\FetchAuthTokenInterface} object or
     *           {@see \Google\ApiCore\CredentialsWrapper} object. Note that when one of these
     *           objects are provided, any settings in $credentialsConfig will be ignored.
     *     @type array $credentialsConfig
     *           Options used to configure credentials, including auth token caching, for the
     *           client. For a full list of supporting configuration options, see
     *           {@see \Google\ApiCore\CredentialsWrapper::build()} .
     *     @type bool $disableRetries
     *           Determines whether or not retries defined by the client configuration should be
     *           disabled. Defaults to `false`.
     *     @type string|array $clientConfig
     *           Client method configuration, including retry settings. This option can be either
     *           a path to a JSON file, or a PHP array containing the decoded JSON data. By
     *           default this settings points to the default client config file, which is
     *           provided in the resources folder.
     *     @type string|TransportInterface $transport
     *           The transport used for executing network requests. May be either the string
     *           `rest` or `grpc`. Defaults to `grpc` if gRPC support is detected on the system.
     *           *Advanced usage*: Additionally, it is possible to pass in an already
     *           instantiated {@see \Google\ApiCore\Transport\TransportInterface} object. Note
     *           that when this object is provided, any settings in $transportConfig, and any
     *           $apiEndpoint setting, will be ignored.
     *     @type array $transportConfig
     *           Configuration options that will be used to construct the transport. Options for
     *           each supported transport type should be passed in a key for that transport. For
     *           example:
     *           $transportConfig = [
     *               'grpc' => [...],
     *               'rest' => [...],
     *           ];
     *           See the {@see \Google\ApiCore\Transport\GrpcTransport::build()} and
     *           {@see \Google\ApiCore\Transport\RestTransport::build()} methods for the
     *           supported options.
     *     @type callable $clientCertSource
     *           A callable which returns the client cert as a string. This can be used to
     *           provide a certificate and private key to the transport layer for mTLS.
     * }
     *
     * @throws ValidationException
     */
    public function __construct(array $options = [])
    {
        $clientOptions = $this->buildClientOptions($options);
        $this->setClientOptions($clientOptions);
    }

    /** Handles execution of the async variants for each documented method. */
    public function __call($method, $args)
    {
        if (substr($method, -5) !== 'Async') {
            trigger_error('Call to undefined method ' . __CLASS__ . "::$method()", E_USER_ERROR);
        }

        array_unshift($args, substr($method, 0, -5));
        return call_user_func_array([$this, 'startAsyncCall'], $args);
    }

    /**
     * Enables a Control on the specified ServingConfig.
     * The control is added in the last position of the list of controls
     * it belongs to (e.g. if it's a facet spec control it will be applied
     * in the last position of servingConfig.facetSpecIds)
     * Returns a ALREADY_EXISTS error if the control has already been applied.
     * Returns a FAILED_PRECONDITION error if the addition could exceed maximum
     * number of control allowed for that type of control.
     *
     * The async variant is {@see ServingConfigServiceClient::addControlAsync()} .
     *
     * @example samples/V2/ServingConfigServiceClient/add_control.php
     *
     * @param AddControlRequest $request     A request to house fields associated with the call.
     * @param array             $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return ServingConfig
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function addControl(AddControlRequest $request, array $callOptions = []): ServingConfig
    {
        return $this->startApiCall('AddControl', $request, $callOptions)->wait();
    }

    /**
     * Creates a ServingConfig.
     *
     * A maximum of 100 [ServingConfig][google.cloud.retail.v2.ServingConfig]s are
     * allowed in a [Catalog][google.cloud.retail.v2.Catalog], otherwise a
     * FAILED_PRECONDITION error is returned.
     *
     * The async variant is
     * {@see ServingConfigServiceClient::createServingConfigAsync()} .
     *
     * @example samples/V2/ServingConfigServiceClient/create_serving_config.php
     *
     * @param CreateServingConfigRequest $request     A request to house fields associated with the call.
     * @param array                      $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return ServingConfig
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function createServingConfig(CreateServingConfigRequest $request, array $callOptions = []): ServingConfig
    {
        return $this->startApiCall('CreateServingConfig', $request, $callOptions)->wait();
    }

    /**
     * Deletes a ServingConfig.
     *
     * Returns a NotFound error if the ServingConfig does not exist.
     *
     * The async variant is
     * {@see ServingConfigServiceClient::deleteServingConfigAsync()} .
     *
     * @example samples/V2/ServingConfigServiceClient/delete_serving_config.php
     *
     * @param DeleteServingConfigRequest $request     A request to house fields associated with the call.
     * @param array                      $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function deleteServingConfig(DeleteServingConfigRequest $request, array $callOptions = []): void
    {
        $this->startApiCall('DeleteServingConfig', $request, $callOptions)->wait();
    }

    /**
     * Gets a ServingConfig.
     *
     * Returns a NotFound error if the ServingConfig does not exist.
     *
     * The async variant is {@see ServingConfigServiceClient::getServingConfigAsync()}
     * .
     *
     * @example samples/V2/ServingConfigServiceClient/get_serving_config.php
     *
     * @param GetServingConfigRequest $request     A request to house fields associated with the call.
     * @param array                   $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return ServingConfig
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function getServingConfig(GetServingConfigRequest $request, array $callOptions = []): ServingConfig
    {
        return $this->startApiCall('GetServingConfig', $request, $callOptions)->wait();
    }

    /**
     * Lists all ServingConfigs linked to this catalog.
     *
     * The async variant is
     * {@see ServingConfigServiceClient::listServingConfigsAsync()} .
     *
     * @example samples/V2/ServingConfigServiceClient/list_serving_configs.php
     *
     * @param ListServingConfigsRequest $request     A request to house fields associated with the call.
     * @param array                     $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return PagedListResponse
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function listServingConfigs(ListServingConfigsRequest $request, array $callOptions = []): PagedListResponse
    {
        return $this->startApiCall('ListServingConfigs', $request, $callOptions);
    }

    /**
     * Disables a Control on the specified ServingConfig.
     * The control is removed from the ServingConfig.
     * Returns a NOT_FOUND error if the Control is not enabled for the
     * ServingConfig.
     *
     * The async variant is {@see ServingConfigServiceClient::removeControlAsync()} .
     *
     * @example samples/V2/ServingConfigServiceClient/remove_control.php
     *
     * @param RemoveControlRequest $request     A request to house fields associated with the call.
     * @param array                $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return ServingConfig
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function removeControl(RemoveControlRequest $request, array $callOptions = []): ServingConfig
    {
        return $this->startApiCall('RemoveControl', $request, $callOptions)->wait();
    }

    /**
     * Updates a ServingConfig.
     *
     * The async variant is
     * {@see ServingConfigServiceClient::updateServingConfigAsync()} .
     *
     * @example samples/V2/ServingConfigServiceClient/update_serving_config.php
     *
     * @param UpdateServingConfigRequest $request     A request to house fields associated with the call.
     * @param array                      $callOptions {
     *     Optional.
     *
     *     @type RetrySettings|array $retrySettings
     *           Retry settings to use for this call. Can be a {@see RetrySettings} object, or an
     *           associative array of retry settings parameters. See the documentation on
     *           {@see RetrySettings} for example usage.
     * }
     *
     * @return ServingConfig
     *
     * @throws ApiException Thrown if the API call fails.
     */
    public function updateServingConfig(UpdateServingConfigRequest $request, array $callOptions = []): ServingConfig
    {
        return $this->startApiCall('UpdateServingConfig', $request, $callOptions)->wait();
    }
}
